#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Wget
#
# This ruby file is a wrapper over wget.
#
# Right now this class offers this functionality:
#
#   (1) download a remote file via wget
#   (2) store the location of this download in a general file.
#
# Usage example:
#
#   Wget.new(ARGV)
#
# =========================================================================== #
# require 'wget'
# =========================================================================== #
require 'wget/toplevel_methods.rb'
require 'wget/version/version.rb'

class Wget # === Wget.new

  require 'fileutils'

  begin
    require 'opn'
  rescue LoadError; end

  begin
    require 'cliner/module'
    include Cliner
  rescue LoadError; end

  begin
    require 'colours'
    include Colours
  rescue LoadError; end
  
  begin
    require 'save_file/module'
    include SaveFile
  rescue LoadError; end

  # ========================================================================= #
  # === BACKUP_DIR
  # ========================================================================= #
  BACKUP_DIR = '/home/Temp/'

  # ========================================================================= #
  # === LAST_DOWNLOADED_FILE
  #
  # The next line tells us where we keep our last downloaded file.
  # ========================================================================= #
  LAST_DOWNLOADED_FILE = ENV['HOME'].to_s+'/LAST_DOWNLOADED_FILE.md'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      download_this = nil,
      run_already   = true
    )
    register_sigint
    reset
    set_url(download_this)
    case run_already.to_s
    when 'dont','do_not','dont_connect','do_not_connect'
      run_already = false
    end
    if block_given?
      yielded = yield
      if yielded.is_a?(Hash) and
         yielded.has_key?(:namespace)
        @pass_this_to_opn = yielded.delete(:namespace)
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    Signal.trap('SIGINT') { e; exit }
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @remote_file_exists
    # ======================================================================= #
    @remote_file_exists = true
    # ======================================================================= #
    # === @pass_this_to_opn
    # ======================================================================= #
    @pass_this_to_opn = nil # Whether to pass something to Opn.opn()
  end

  # ========================================================================= #
  # === rds
  # ========================================================================= #
  def rds(i)
    i.squeeze('/')
  end

  # ========================================================================= #
  # === saving_last_downloaded_file
  # ========================================================================= #
  def saving_last_downloaded_file
    # ======================================================================= #
    # If the file was successfully downloaded, we will log this.
    # ======================================================================= #
    if File.exist?(local_target?)
      log_this_file_was_downloaded(local_target?)
    end
  end

  # ========================================================================= #
  # === log_this_file_was_downloaded
  # ========================================================================= #
  def log_this_file_was_downloaded(i)
    opnn; e "Next storing #{simp(i)}"
    opnn; e 'in '+sfile(LAST_DOWNLOADED_FILE)
    # ======================================================================= #
    # As of 17.09.2014 we also add the time:
    # ======================================================================= #
    i << " # #{Time.now}"
    save_what_into(i, LAST_DOWNLOADED_FILE)
  end

  # ========================================================================= #
  # === consider_making_a_backup
  # ========================================================================= #
  def consider_making_a_backup
    if remote_file_exists? and File.exist?(local_file?)
      opnn; e "We will first backup the existing file at `"\
              "#{sfile(local_file?)}`."
      backup_existing_file(local_file?)
    end
  end

  # ========================================================================= #
  # === backup_existing_file
  # ========================================================================= #
  def backup_existing_file(i = local_target?)
    new_target = BACKUP_DIR+File.basename(i)
    opnn; e 'Now moving existing file from '+sfile(i)+
            ' to '+sfile(new_target)+'.'
    FileUtils.mv(i, new_target)
    File.delete(i) if File.exist? i
  end

  # ========================================================================= #
  # === local_target?
  # ========================================================================= #
  def local_target?
    rds(Dir.pwd+'/'+File.basename(@url))
  end; alias local_file? local_target? # === local_file?

  # ========================================================================= #
  # === consider_downloading_original_file_via_wget
  #
  # Simply use wget to download a remote file, but only if it actually
  # exists.
  # ========================================================================= #
  def consider_downloading_original_file_via_wget
    if remote_file_exists?
      _ = "wget #{@url}"
      opnn; e _
      system(_)
    end
  end

  # ========================================================================= #
  # === download_last_url
  #
  # This will try to download the last saved URL.
  # ========================================================================= #
  def download_last_url
    _ = LAST_DOWNLOADED_FILE
    if File.exist? _
      where = File.readlines(_).first
      # ======================================================================= #
      # where may include a '#' character such as in
      # "/Depot/jj/pygtksourceview-2.3.0.tar.bz2 # 2015-01-05 02:37:12 +0100"
      # We will eliminate this part next.
      # ======================================================================= #
      where = where[0, where.index('#')]
      where = where.strip
      return where
    else
      e 'The file at '+sfile(_)+' does not exist.'
    end
  end; alias downloaded_where? download_last_url # === downloaded_where?

  # ========================================================================= #
  # === url?
  # ========================================================================= #
  def url?
    @url
  end; alias input? url? # === input?

  # ========================================================================= #
  # === remote_file_exists?
  # ========================================================================= #
  def remote_file_exists?
    @remote_file_exists
  end; alias remote_file_exists remote_file_exists? # === remote_file_exists

  # ========================================================================= #
  # === Wget.remote_file_exists?
  # ========================================================================= #
  def self.remote_file_exists?(i)
    _ = Wget.new(i, :dont_run_yet)
    _.check_if_remote_file_exists
    return _.remote_file_exists
  end

  # ========================================================================= #
  # === try_to_report_where_we_downloaded_the_file
  # ========================================================================= #
  def try_to_report_where_we_downloaded_the_file
    if downloaded_where?.include? input?
      opnn; e 'We downloaded into: '+sfancy(downloaded_where?)
    end
  end

  # ========================================================================= #
  # === check_if_remote_file_exists
  #
  # This method will set the variable @remote_file_exists appropriately.
  # ========================================================================= #
  def check_if_remote_file_exists
    @remote_file_exists = Wget.check_if_remote_file_exists(url?)
  end

  # ========================================================================= #
  # === url?
  # ========================================================================= #
  def url?
    @url
  end

  # ========================================================================= #
  # === set_url
  # ========================================================================= #
  def set_url(i = nil)
    i = i.first if i.is_a? Array
    if i.nil?
      opnn; e 'Wget: We are missing an URL.'
      opnn; e 'Usage: wget [URL]'
      exit
    end
    case i # case tag
    # ======================================================================= #
    # === wget --query
    # ======================================================================= #
    when 'QUERY?','?','--query'
      opnn; e 'The last downloaded URL is at '+sfancy(download_last_url)+'.'
      exit
    when 'LAST'
      i = download_last_url
    end
    i = i.to_s.dup
    i.strip!
    if i.end_with? '/' or i.end_with? ':'
      i.chop!
    end
    @url = i
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn(optional_pass_this = @pass_this_to_opn)
    if optional_pass_this
      Opn.opn(optional_pass_this)
    else
      Opn.opn
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_if_remote_file_exists
    consider_making_a_backup
    consider_downloading_original_file_via_wget
    saving_last_downloaded_file # We save the last downloaded file, to make further use of that information.
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = Wget.new(ARGV)
  _.try_to_report_where_we_downloaded_the_file
end # wgetwrapper http://www.bitcoin.pdf/